#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import url
from bigbuild import views, feeds, sitemaps


urlpatterns = [
    # Index redirect
    url(r'^$', views.IndexRedirectView.as_view(), name="index-redirect"),

    # Pages
    url(r'^projects/$', views.PageListView.as_view(), name='page-list'),
    url(
        r'^projects/(?P<slug>[-\w]+)/$',
        views.PageDetailView.as_view(),
        name='page-detail'
    ),

    # Static assets
    url(
        r'^projects/(?P<slug>[-\w]+)/static/(?P<path>.*)$',
        views.page_static_serve,
        name='page-detail-static'
    ),

    # Machine-readable feeds
    url(
        r'^projects/sitemap.xml$',
        sitemaps.SitemapView.as_view(),
        name='sitemap'
    ),
    url(
        r'^projects/google-news-sitemap.xml$',
        sitemaps.GoogleNewsSitemapView.as_view(),
        name='google-news-sitemap'
    ),
    url(
        r'^projects/feeds/latest.xml$',
        feeds.LatestPages(),
        name="feeds-latest"
    ),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import csv
import json
import yaml
import codecs
import shutil
import logging
import frontmatter
from django.conf import settings
from bigbuild.models import BasePage
from bigbuild import get_page_directory
from bigbuild.exceptions import BadMetadata
from django.template import Engine, Context
logger = logging.getLogger(__name__)


class Page(BasePage):
    """
    A custom page published via static.latimes.com
    """
    @property
    def base_directory(self):
        """
        Returns base directory for pages.
        """
        return get_page_directory()

    @property
    def retired_directory_path(self):
        """
        Returns the path where this page would be retired, if it were retired.
        """
        return os.path.join(settings.RETIRED_DIR, self.slug)

    @property
    def retired_directory_exists(self):
        """
        Tests whether a retired directory for this slug already exists.
        """
        return os.path.exists(self.retired_directory_path)

    def create_directory(
        self,
        force=False,
        index_template_name='bigbuild/pages/default_index.html',
        index_template_context={},
    ):
        """
        Creates a new directory for the page.

        Returns the path to the directory that has been created, which is
        the same as the self.directory_path property.

        Throws an error if the directory already exists. You can force it
        to overwrite a pre-existing directory by submitting the force keyword
        argument as true.
        """
        # Check if the directory exists already and act accordingly
        if self.directory_exists:
            if force:
                shutil.rmtree(self.directory_path)
            else:
                raise ValueError(
                    "Page directory already exists at %s" % self.directory_path
                )

        # Make the directory
        os.mkdir(self.directory_path)

        # Add the index template
        self.write_index(
            template_name=index_template_name,
            template_context=index_template_context,
        )

        # Create the metadata configuration file
        self.write_frontmatter()

        # Create the ./static/ directory for extra stuff
        self.write_static()

        # Create the checklist file
        self.write_checklist()

        # Return the directory path
        return self.directory_path

    def write_index(self, template_name, template_context={}):
        """
        Creates index.html in the page directory.
        """
        template = self.get_template(template_name)
        html = template.render(Context(template_context))
        with codecs.open(self.index_path, 'w', 'utf-8') as f:
            f.write(html)

    def write_frontmatter(self):
        """
        Creates metadata.yaml in the page directory.
        """
        frontmatter.dump(self, self.frontmatter_path)

    def write_static(self):
        """
        Creates a ./static/ subdirectory within the page directory.
        """
        os.mkdir(self.static_path)

        css_path = os.path.join(self.static_path, 'style.css')
        css = self.get_template("bigbuild/pages/style.css-tpl").source
        with codecs.open(css_path, 'w', 'utf-8') as f:
            f.write(css)

        js_path = os.path.join(self.static_path, 'app.js')
        js = self.get_template("bigbuild/pages/app.js-tpl").source
        with codecs.open(js_path, 'w', 'utf-8') as f:
            f.write(js)

    def write_checklist(self):
        """
        Creates checklist.md in the page directory.
        """
        template = self.get_template("bigbuild/pages/checklist.md-tpl")
        md = template.render(Context(dict(object=self)))
        with codecs.open(self.checklist_path, 'w', 'utf-8') as f:
            f.write(md)

    def get_template(self, name):
        """
        Returns a Django template of the provided name ready to render.
        """
        engine = Engine.get_default()
        return engine.get_template(name)

    def sync_frontmatter(self):
        super(Page, self).sync_frontmatter()
        with codecs.open(self.frontmatter_path) as f:
            # Parse the YAML with confidence since the super call above
            # has already weeded out any errors
            post = frontmatter.load(f)
            # Loop through any data files
            self.data = {}
            for key, path in post.metadata.get('data', {}).items():
                # Generate what the path to the file ought to be
                p = os.path.join(self.data_path, path)
                # Toss an error if it doesn't exist
                if not os.path.exists(p):
                    raise BadMetadata("Data file could not be found at %s" % p)
                # Open the file
                with codecs.open(p, 'r') as f:
                    # If it's a CSV file open it that way...
                    if p.endswith(".csv"):
                        self.data[key] = list(csv.DictReader(f))
                    # If it's a JSON file open it this way ...
                    elif p.endswith(".json"):
                        self.data[key] = json.load(f)
                    # If it's a YAML file open it t'other way ...
                    elif (p.endswith(".yml") or p.endswith(".yaml")):
                        self.data[key] = yaml.load(f)
                    # If it's none of those throw an error.
                    else:
                        raise BadMetadata(
                            "Data file at %s not recognizable type" % path
                        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import yaml
import shutil
import logging
import validictory
import frontmatter
from datetime import datetime
from django.conf import settings
from django.utils import timezone
from greeking import latimes_ipsum
from django.test import RequestFactory
from bigbuild.exceptions import BadMetadata
from django.template import Engine, RequestContext
from django.template.defaultfilters import slugify
logger = logging.getLogger(__name__)


class BasePage(object):
    """
    An abstract base class with methods shared by all page classes.
    """
    def __init__(
        self,
        slug,
        headline="",
        byline="",
        description="",
        pub_date=None,
        published=False,
        show_in_feeds=True,
        content='',
        extra={},
        data={}
    ):
        # The metadata
        self.slug = slugify(slug)
        self.headline = headline
        self.byline = byline
        self.description = description
        self.image_url = ""
        self.pub_date = pub_date or datetime.now().replace(
            second=0,
            microsecond=0
        )
        self.published = published
        self.show_in_feeds = show_in_feeds
        # The content
        self.content = content
        # Any extras
        self.extra = extra
        # Any structured data
        self.data = data

    def __repr__(self):
        return "<%s %s>" % (self.__class__.__name__, self.__str__())

    def __str__(self):
        return str(self.slug)

    def to_json(self):
        d = self.__dict__
        d['pub_date'] = str(d['pub_date'])
        return d

    @property
    def metadata(self):
        """
        Returns the structured metadata associated with this page.

        For the purpose of deconstructing this object to Jekyll frontmatter.
        """
        d = dict([
            ('slug', str(self.slug)),
            ('headline', self.headline),
            ('byline', self.byline),
            ('description', self.description),
            ('image_url', self.image_url),
            ('pub_date', self.pub_date),
            ('published', self.published),
            ('show_in_feeds', self.show_in_feeds),
        ])
        if self.extra:
            d['extra'] = self.extra
        if self.data:
            d['data'] = self.data
        return d

    def get_absolute_url(self):
        return '/projects/%s/' % self.slug

    def get_static_url(self):
        return os.path.join(self.get_absolute_url(), 'static') + "/"

    def delete(self):
        """
        Delete the page directory.
        """
        shutil.rmtree(self.directory_path)

    def build(self):
        """
        Builds this page via the PageDetailView.
        """
        from bigbuild.views import PageDetailView
        view = PageDetailView()
        view.build_object(self)

    @property
    def directory_path(self):
        """
        Returns the directory path where this page will exist.
        """
        return os.path.join(self.base_directory, self.slug)

    @property
    def directory_exists(self):
        """
        Tests whether the page directory for this object exists.

        Returns True or False
        """
        return os.path.exists(self.directory_path)

    @property
    def index_path(self):
        """
        Returns the index.html path where this page will be marked up.
        """
        return os.path.join(self.directory_path, 'index.html')

    @property
    def frontmatter_path(self):
        """
        Returns the metadata.md path where this page will be configured.
        """
        return os.path.join(self.directory_path, 'metadata.md')

    @property
    def static_path(self):
        """
        Returns the path to the static subdirectory for extra files.
        """
        return os.path.join(self.directory_path, 'static')

    @property
    def checklist_path(self):
        """
        Returns the checklist.md path where the checklist will be stored.
        """
        return os.path.join(self.directory_path, 'checklist.md')

    @property
    def data_path(self):
        """
        Returns the path to the data subdirectory for structured data files.
        """
        return os.path.join(self.directory_path, 'data')

    def sync_frontmatter(self):
        """
        Reads in the frontmatter from metadata.yaml and syncs it with
        the object.
        """
        with open(self.frontmatter_path) as f:
            # Parse the YAML
            try:
                post = frontmatter.load(f)
            except (yaml.scanner.ScannerError, yaml.parser.ParserError) as e:
                raise BadMetadata(
                    "THE METADATA.MD FILE FOR '%s' IS BROKEN! \n" % self.slug +
                    "Here's a clue about what's wrong: \n" +
                    "%s" % e
                )

            # Set the basic frontmatter metadata
            self.slug = post.metadata['slug']
            self.headline = post.metadata['headline']
            self.byline = post.metadata['byline']
            self.description = post.metadata['description']
            self.image_url = post.metadata['image_url']
            self.pub_date = post.metadata['pub_date']
            self.published = post.metadata['published']
            self.show_in_feeds = post.metadata['show_in_feeds']

            # Attach any extra stuff
            try:
                self.extra = post.metadata['extra']
            except KeyError:
                self.extra = {}

            # Render the content as a Django template
            engine = Engine.get_default()
            template = engine.from_string(post.content)
            context = RequestContext(
                RequestFactory().get(self.get_absolute_url()),
                {
                    "object": self,
                    'STATIC_URL': self.get_static_url()
                }
            )
            content = template.render(context)
            self.content = content

    def is_metadata_valid(self):
        """
        Tests if the metadata is valid and returns True or False.
        """
        schema = {
            "type": "object",
            "properties": {
                'slug': {"type": "string"},
                'headline': {"type": "string", "blank": True},
                'byline': {"type": "string", "blank": True},
                'description': {"type": "string", "blank": True},
                'image_url': {"type": "string", "blank": True},
                'pub_date': {"type": "any"},
                'published': {"type": "boolean"},
                'show_in_feeds': {"type": "boolean"},
            }
        }
        try:
            validictory.validate(self.metadata, schema)
            if not isinstance(self.pub_date, datetime):
                return False
            return True
        except:
            return False

    def has_recommended_metadata(self):
        """
        Tests if the metadata has the fields we recommend are present
        for every page have been filled.

        Returns True or False.
        """
        recommended_fields = ['headline', 'description', 'byline', 'image_url']
        md = self.metadata
        boilerplate = latimes_ipsum.get_story()
        for f in recommended_fields:
            # If it's an empty string cry foul
            if not md.get(f, ''):
                return False
            # Same thing if the value is from our boilerplate
            if md[f] == getattr(boilerplate, f):
                return False
        return True

    @property
    def pub_status(self):
        """
        Returns the publication status as a string.

        There are three values:

            - live
            - working
            - pending
        """
        # Drop all the working ones right away
        if self.published is not True:
            return 'working'
        # If it is published but has a future date, that means its pending
        if timezone.make_aware(self.pub_date) > timezone.now():
            return 'pending'
        # Otherwise it should go live
        return 'live'

    def is_live(self):
        """
        Tests if the object should be published on the live production site.

        Returns True or False.
        """
        status_dict = {
            'live': True,
            'pending': False,
            'working': False
        }
        return status_dict[self.pub_status]

    def should_build(self):
        """
        Tests if the object should be published in the current environment.

        Returns True or False
        """
        # Get the publication status of the current environment
        status = getattr(settings, 'PAGE_PUBLICATION_STATUS', 'working')
        # If we are in 'working' mode everything gets built
        if status == 'working':
            return True
        # Otherwise default to the object's status
        return self.is_live()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild.models import Page
from greeking import latimes_ipsum
from django.template.defaultfilters import linebreaks
from bigbuild.templatetags.bigbuild_tags import dropcap
from bigbuild.management.commands.createpage import Command as BaseCommand


class Command(BaseCommand):
    help = 'Creates a new page directory for a custom "big build" story'
    INDEX_TEMPLATE_NAME = 'bigbuild/pages/bigbuild_index.html'

    def create_page(self, slug):
        obj = latimes_ipsum.get_story()
        return Page(
            slug,
            headline=obj.headline,
            description=obj.description,
            byline=obj.byline,
            content=linebreaks(dropcap(obj.content)),
            extra=dict(
                heroic_image=dict(
                    src='http://placehold.it/2000x800',
                    caption='',
                    credit=''
                ),
                credits='Credits TK',
                more_like_this=[o.__dict__ for o in latimes_ipsum.get_related_items()],
            )
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Slimmed down subclasses of django-compressor internals that aim to work
without django.contrib.staticfiles installed.
"""
import codecs
from django.conf import settings
from compressor.base import Compressor
from compressor.js import JsCompressor
from compressor.css import CssCompressor
from django.template import Template, Context
from compressor.exceptions import UncompressableFileError


class SimpleCompressor(Compressor):
    """
    A simplification of django-compressor's standard compression class.

    The aim is to work without django.contrib.staticfiles installed.
    """
    def get_filename(self, basename):
        """
        A simplification of the standard method to remove a hack around
        the staticfiles caching system we don't want.
        """
        # Get the filename from our storage backend
        filename = self.storage.path(basename)

        # If it exists, return it
        if self.storage.exists(basename):
            return filename

        # If it doesn't, raise an exception
        raise UncompressableFileError(
            "'%s' could not be found in the COMPRESS_ROOT '%s'%s" %
            (basename, settings.COMPRESS_ROOT,
             self.finders and " or with staticfiles." or "."))

    def precompile(
        self,
        content,
        kind=None,
        elem=None,
        filename=None,
        charset=None,
        **kwargs
    ):
        """
        An expansion of the standard method that will halt precompilation
        when compression is off.
        """
        # If compression is off, skip it
        if not kind or not settings.COMPRESS_ENABLED:
            return False, content

        # Clear out the filename settings so the compilers will use
        # the rendered in-memory string and a temporary file instead.
        kwargs.pop("basename", "")
        filename = None

        # Otherwise compile away as usual
        return super(SimpleCompressor, self).precompile(
            content,
            kind,
            elem,
            filename,
            charset,
            **kwargs
        )

    def get_filecontent(self, filename, charset):
        """
        A custom override that renders file content as a Django template
        with the page context included.

        This allows for metadata from the page object to be included in
        static files.
        """
        if charset == 'utf-8':
            # Removes BOM
            charset = 'utf-8-sig'
        with codecs.open(filename, 'r', charset) as fd:
            # All the custom bits are right here
            content = fd.read()
            template = Template(content)
            context = Context(self.context)
            rendered_content = template.render(context)
            return rendered_content


class SimpleCssCompressor(CssCompressor, SimpleCompressor):
    pass


class SimpleJsCompressor(JsCompressor, SimpleCompressor):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild.models import PageList
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Tests if page directories are valid'

    def handle(self, *args, **options):
        page_list = PageList()
        self.stdout.write(
            self.style.SUCCESS('All %s pages are valid' % len(page_list))
        )



import os
from django.conf import settings
default_app_config = 'bigbuild.apps.BigbuildConfig'


def get_page_directory():
    """
    Returns the PAGE_DIR where dynamic pages are configured.
    """
    # Return the PAGE_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'PAGE_DIR',
        os.path.join(settings.BASE_DIR, 'pages')
    )
    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)
    # Return the path
    return path


def get_retired_directory():
    """
    Returns the RETIRED_DIR where retired pages are configured.
    """
    # Return the RETIRED_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'RETIRED_DIR',
        os.path.join(settings.BASE_DIR, '.retired')
    )
    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)
    # Return the path
    return path
